@@ -23,6 +23,7 @@
 #  include "pdf-properties-display.h"
 #  include <bonobo.h>
 #  include <libgnomevfs/gnome-vfs-uri.h>
+#  include "eog-hig-dialog.h"
 #include "gpdf-g-switch.h"
 #include "gpdf-persist-stream.h"
 #include "gpdf-persist-file.h"
@@ -480,6 +481,7 @@ gc_set_page_cb (GPdfPageControl *page_control, gint page, GPdfControl *control)
 	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (control));
 
 	gpdf_view_goto_page (GPDF_VIEW (control->priv->gpdf_view), page);
+	gpdf_view_scroll_to_top (GPDF_VIEW (control->priv->gpdf_view));
 }
 
 static void
@@ -726,7 +728,8 @@ persist_stream_set_pdf_cb (GPdfPersistStream *persist_stream,
 }
 
 static void
-persist_file_set_pdf_cb (GPdfPersistFile *persist_file, gpointer user_data)
+persist_file_loading_finished_cb (GPdfPersistFile *persist_file,
+				  gpointer user_data)
 {
 	GPdfControl *control;
 	const char *uri;
@@ -759,6 +762,39 @@ persist_file_set_pdf_cb (GPdfPersistFile *persist_file, gpointer user_data)
 	gpdf_control_pdf_doc_changed (control);
 }
 
+static void
+persist_file_loading_failed_cb (GPdfPersistFile *persist_file,
+				const char *message,
+				gpointer user_data)
+{
+	GPdfControl *control;
+	GtkWidget *dlg;
+	gchar *primary;
+
+	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (user_data));
+	
+	control = GPDF_CONTROL (user_data);			  
+
+	control->priv->title = g_strdup (_("PDF Document"));
+	control->priv->pdf_doc = NULL;
+		
+	primary = g_strdup_printf (
+		_("Loading of %s failed."),
+		gpdf_persist_file_get_current_uri (persist_file));	
+
+	dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_ERROR, primary,
+				  message, FALSE);
+	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
+	g_signal_connect_swapped (G_OBJECT (dlg), "response",
+				  G_CALLBACK (gtk_widget_destroy), dlg);
+	bonobo_control_set_transient_for (BONOBO_CONTROL (control),
+					  GTK_WINDOW (dlg), NULL);
+	gtk_widget_show (dlg);
+	g_free (primary);
+
+	gpdf_control_pdf_doc_changed (control);
+}
+
 static void
 gpdf_control_destroy (BonoboObject *object)
 {
@@ -832,8 +868,12 @@ gpdf_control_setup_persist (GPdfControl *control)
 			  G_CALLBACK (persist_stream_set_pdf_cb),
 			  control);
 	g_signal_connect (G_OBJECT (control->priv->persist_file),
-			  "set_pdf",
-			  G_CALLBACK (persist_file_set_pdf_cb),
+			  "loading_finished",
+			  G_CALLBACK (persist_file_loading_finished_cb),
+			  control);
+	g_signal_connect (G_OBJECT (control->priv->persist_file),
+			  "loading_failed",
+			  G_CALLBACK (persist_file_loading_failed_cb),
 			  control);
 }
 
