@@ -214,6 +214,8 @@ pdf_doc_process_properties (PDFDoc *pdf_doc, GObject *target)
 
 	g_object_set (target,
 		      "optimized",
+		      /* Yes/No will be displayed in the pdf properties dialog
+			 in a table as "Optimized: No" */
 		      pdf_doc->isLinearized () ? _("Yes") : _("No"),
 		      NULL);
 }
